@@ -55,6 +55,8 @@ get_error_display_message (GError *error)
         return _("The specified contact is not valid");
       case TP_ERROR_EMERGENCY_CALLS_NOT_SUPPORTED:
         return _("Emergency calls are not supported on this protocol");
+      case TP_ERROR_INSUFFICIENT_BALANCE:
+        return _("You don't have enough credit in order to place this call");
     }
 
   return _("There was an error starting the call");
@@ -132,6 +134,7 @@ create_streamed_media_channel_cb (GObject *source,
     }
 }
 
+#ifdef HAVE_CALL
 static void
 create_call_channel_cb (GObject *source,
     GAsyncResult *result,
@@ -163,8 +166,9 @@ create_call_channel_cb (GObject *source,
       NULL);
 }
 
-void
-empathy_call_new_with_streams (const gchar *contact,
+/* Try to request a Call channel and fallback to StreamedMedia if that fails */
+static void
+call_new_with_streams (const gchar *contact,
     TpAccount *account,
     gboolean initial_audio,
     gboolean initial_video,
@@ -173,45 +177,101 @@ empathy_call_new_with_streams (const gchar *contact,
   GHashTable *call_request, *streamed_media_request;
   TpAccountChannelRequest *call_req, *streamed_media_req;
 
+  /* Call */
   call_request = empathy_call_create_call_request (contact,
       initial_audio,
       initial_video);
 
+  call_req = tp_account_channel_request_new (account, call_request, timestamp);
+
+  g_hash_table_unref (call_request);
+
+  /* StreamedMedia */
   streamed_media_request = empathy_call_create_streamed_media_request (
       contact, initial_audio, initial_video);
 
-  call_req = tp_account_channel_request_new (account, call_request, timestamp);
   streamed_media_req = tp_account_channel_request_new (account,
       streamed_media_request,
       timestamp);
 
+  g_hash_table_unref (streamed_media_request);
+
   tp_account_channel_request_create_channel_async (call_req,
       EMPATHY_CALL_BUS_NAME, NULL,
       create_call_channel_cb,
       streamed_media_req);
 
-  g_hash_table_unref (call_request);
-  g_hash_table_unref (streamed_media_request);
   g_object_unref (call_req);
 }
 
+#else /* HAVE_CALL */
+
+static void
+sm_new_with_streams (const gchar *contact,
+    TpAccount *account,
+    gboolean initial_audio,
+    gboolean initial_video,
+    gint64 timestamp)
+{
+  GHashTable *streamed_media_request;
+  TpAccountChannelRequest *streamed_media_req;
+
+  /* StreamedMedia */
+  streamed_media_request = empathy_call_create_streamed_media_request (
+      contact, initial_audio, initial_video);
+
+  streamed_media_req = tp_account_channel_request_new (account,
+      streamed_media_request,
+      timestamp);
+
+  g_hash_table_unref (streamed_media_request);
+
+  tp_account_channel_request_create_channel_async (streamed_media_req,
+      EMPATHY_AV_BUS_NAME, NULL, create_streamed_media_channel_cb, NULL);
+
+  g_object_unref (streamed_media_req);
+}
+#endif /* HAVE_CALL */
+
 void
-empathy_call_set_stream_properties (GstElement *element)
+empathy_call_new_with_streams (const gchar *contact,
+    TpAccount *account,
+    gboolean initial_audio,
+    gboolean initial_video,
+    gint64 timestamp)
+{
+#ifdef HAVE_CALL
+  call_new_with_streams (contact, account, initial_audio, initial_video,
+      timestamp);
+#else
+  sm_new_with_streams (contact, account, initial_audio, initial_video,
+      timestamp);
+#endif
+}
+
+void
+empathy_call_set_stream_properties (GstElement *element,
+  gboolean echo_cancellation)
 {
   GstStructure *props;
   GSettings *gsettings_call;
-  gboolean echo_cancellation;
+  gboolean echo_cancellation_setting;
 
   gsettings_call = g_settings_new (EMPATHY_PREFS_CALL_SCHEMA);
 
-  echo_cancellation = g_settings_get_boolean (gsettings_call,
+  echo_cancellation_setting = g_settings_get_boolean (gsettings_call,
       EMPATHY_PREFS_CALL_ECHO_CANCELLATION);
 
+  DEBUG ("Echo cancellation: element allowed: %s, user enabled: %s",
+    echo_cancellation ? " yes" : "no",
+    echo_cancellation_setting ? " yes" : "no");
+
+
   props = gst_structure_new ("props",
       PA_PROP_MEDIA_ROLE, G_TYPE_STRING, "phone",
       NULL);
 
-  if (echo_cancellation)
+  if (echo_cancellation && echo_cancellation_setting)
     {
       gst_structure_set (props,
           "filter.want", G_TYPE_STRING, "echo-cancel",
