@@ -680,8 +680,7 @@ location_update (EmpathyIndividualWidget *self)
 
 #ifdef HAVE_LIBCHAMPLAIN
   if ((g_hash_table_lookup (location, EMPATHY_LOCATION_LAT) != NULL) &&
-      (g_hash_table_lookup (location, EMPATHY_LOCATION_LON) != NULL) &&
-      !(priv->flags & EMPATHY_INDIVIDUAL_WIDGET_FOR_TOOLTIP))
+      (g_hash_table_lookup (location, EMPATHY_LOCATION_LON) != NULL))
     {
       /* Cannot be displayed in tooltips until Clutter-Gtk can deal with such
        * windows */
@@ -1675,6 +1674,17 @@ individual_grid_destroy (EmpathyIndividualWidget *self)
 
   gtk_container_remove (GTK_CONTAINER (priv->vbox_individual),
       GTK_WIDGET (priv->individual_grid));
+
+#ifdef HAVE_LIBCHAMPLAIN
+  if (priv->map_view_embed != NULL)
+    {
+      gtk_container_remove (GTK_CONTAINER (priv->viewport_map),
+          priv->map_view_embed);
+
+      priv->map_view_embed = NULL;
+    }
+#endif
+
   priv->individual_grid = NULL;
 }
 
